--[[ neo chaos hud

INFO:
	- Hud elements are rendered from the top-left most pixel.
	- We are aiming for bottom center of screen. Ergo, calculate from lowest resolution, account for Y-postion in all resolutions, go from there.

	- 3F3F3F = Neutral/No Effect
	- 6F0000 = Negative Effect
	- 005F00 = Positive Effect

]]

local function tablelength(T)
    local count = 0
    for _ in pairs(T) do count = count + 1 end
    return count
end

local function IsMatchMode()
    local header = mapheaderinfo[gamemap].typeoflevel
    if (header & TOL_MATCH) or (header & TOL_TAG) or (header & TOL_CTF) then
        return true
    end

    return false
end

local function chaotic(v, player)
    if not player.chaotic then return end
    if not v.intialized then
        v.ytime = 215
        v.XCenterVirtual = 320/2
        v.intialized = true
    end

    local lengthofthing = 0

    local YMax = 175

    if IsMatchMode() then
        YMax = 155
    end

    if player.chaoticsettings.position == "top" then
        YMax = 8 -- seems a lot smaller than one'd think...
    end

    local TotalEffects = 0

    for index, value in ipairs(player.chaoticeffects) do
        if not value.hide then
            TotalEffects = $ + 1
        end
    end

    TotalEffects = $ + 2

    --local TotalEffects = tablelength(player.chaoticeffects) + 2 -- but why??? why does it work this way????????
    -- info: sprites are 17x17, spacing in-between is 3
    local predictedlength = (TotalEffects*17) + ((TotalEffects-1)*3)
    local screenwidth = 320

    -- player.x + ((player.width/2) - (Meter.width/2))
    local XCenter = ((screenwidth/2) - (predictedlength/2))

    if v.XCenterVirtual != XCenter then
        if v.XCenterVirtual < XCenter then
            v.XCenterVirtual = $ + 1
        elseif v.XCenterVirtual > XCenter then
            v.XCenterVirtual = $ - 1
        end
    end

    local EXTRAFLAG = V_SNAPTOBOTTOM
    if player.chaoticsettings.position == "top" then
        EXTRAFLAG = V_SNAPTOTOP
    end

    for index, value in ipairs(player.chaoticeffects) do
        if not value.hide then -- skips if the effect should be hidden
            local invisiblink = false
            lengthofthing = $ + 1
            local spritz = value.sprite
            if not value.notimer and (value.timer < 3*TICRATE and (value.timer % 2 == 0)) then
                local blink
                if value.effect == "good" then
                    blink = "CEIC90"
                elseif value.effect == "neutral" then
                    blink = "CEICA0"
                else -- bad
                    blink = "CEICW0"
                end
                spritz = blink
                if value.invisibleblink then
                    invisiblink = true
                end
            elseif invisiblink then
                invisiblink = false
            end

            local ytime = 215
            if player.chaoticsettings.position == "top" then
                ytime = -30
            end
            if value.ytime then
                ytime = value.ytime
            end

            if not invisiblink then v.drawScaled((v.XCenterVirtual*FRACUNIT)+((20*lengthofthing)*FRACUNIT), (ytime)*FRACUNIT, FRACUNIT, v.cachePatch(spritz), V_PERPLAYER|EXTRAFLAG) end
            if not value.notimer then v.drawString(v.XCenterVirtual+1+(20*lengthofthing), ytime+1, value.timer/TICRATE + 1, V_PERPLAYER|V_ALLOWLOWERCASE|V_30TRANS|EXTRAFLAG, "thin") end
        end
    end

    --[[if player.chaotic.StatusEffect then
        -- reach 180
        --local y = 210 - (player.chaotic.StatusHUDTimer*2)

        if player.chaoticsettings.position == "top" then
            v.ytime = $ + 2
            if v.ytime > YMax then
                v.ytime = YMax
            end
        else
            v.ytime = $ - 2
            if v.ytime < YMax then
                v.ytime = YMax
            end
        end

        v.drawScaled((58*FRACUNIT)+(20*(1+lengthofthing))*FRACUNIT, (v.ytime)*FRACUNIT, FRACUNIT, v.cachePatch(player.chaotic.StatusHUD), V_PERPLAYER)
    else
        v.ytime = 220
    end]]

    if player.chaotictext then
        for index, value in ipairs(player.chaotictext) do
            local yoffset = 0
            local realy = 165-(9*(index-1))
            if player.chaoticsettings.position == "top" then
                yoffset = 0 - player.chaotic.HUDMoveUp
                realy = 28+(9*(index-1))
            else
                yoffset = 0 + player.chaotic.HUDMoveUp
                if IsMatchMode() then
                    yoffset = $ + 20
                end
            end
            local FLAGZ = value.flags
            if FLAGZ != nil then
                FLAGZ = $|EXTRAFLAG
            end
            v.drawString(160, realy-yoffset, value.text, FLAGZ, "thin-center")
        end
    end

    if player.chaotic.ReadHelp then
        v.drawString(160, 103, "Welcome to SRB2 Chaotic Edition!", V_ALLOWLOWERCASE|V_30TRANS, "thin-center")
        v.drawString(160, 113, "Please pause & open the console (press\x87 " + input.keyNumToName(input.gameControlToKeyNum(GC_CONSOLE)) + "\x80) and type\x87 chaotic help\x80!", V_ALLOWLOWERCASE|V_30TRANS, "thin-center")
    else
        -- popup text to show version number n junk
        local textpopupcolor = "\x85"
        if tablelength(player.chaoticsettings.blacklist) == 0 then
            textpopupcolor = "\x83"
        elseif tablelength(player.chaoticsettings.blacklist) <= 2 then
            textpopupcolor = "\x82"
        end

        if player.chaotic.TextPopUpFade < 10 then
            v.drawString(160, 188, textpopupcolor + "Chaotic Edition v" + SRB2CEversion, V_ALLOWLOWERCASE|V_SNAPTOBOTTOM|player.chaotic.TextPopUpFade<<V_ALPHASHIFT, "thin-center")
        end
    end
end

hud.add(chaotic, "game")

local function timerhud(v,player)
    if player.chaoticsettings.timerbar == "off" then
        return
    end
    if (((player.pflags & PF_FINISHED) or stoppedclock) and player.chaoticsettings.afterfinish == "off") or player.playerstate == PST_DEAD then
        return
    end
    local width = 320
    local height = 3
    local x = 0
    local y = 200-height
    local SNAP = V_SNAPTOBOTTOM

    local MaxTime
    if not player.chaotic.WaitTime then
        MaxTime = 10*TICRATE
    else
        MaxTime = player.chaotic.WaitTime
    end


    if player.chaoticsettings.position == "top" then
        y = 0
        SNAP = V_SNAPTOTOP
    end

    if not v.color then
        v.color = 0
    end

    local screenwidth = 320
    local XCenter = ((screenwidth/2) - (width/2))

    -- player.chaotic.timer > player.chaotic.WaitTime

    local soup = FixedDiv(player.chaotic.timer, MaxTime)
    local souper = FixedMul(width, soup)

    if player.chaoticsettings.rainbow == "on" then
        local tlength = tablelength(rainbowcolors)
        v.color = $ + 1
        if v.color > tlength then
            v.color = 1
        end
        v.colorset = tonumber(rainbowcolors[v.color])
    else
        v.colorset = 36
    end

    v.drawFill(XCenter, y, width, height, 31|V_PERPLAYER|SNAP)
    v.drawFill(XCenter, y, souper, height, v.colorset+5|V_PERPLAYER|SNAP) -- filling timer bar, before the two pixel gap
    v.drawFill(XCenter, y, souper-2, height, v.colorset|V_PERPLAYER|SNAP) -- the actual color for the timer bar
end

hud.add(timerhud, "game")